/*
* This file is part of SkunkDB
* Copyright (c) 2001 Gianni Tedesco
*
*    SkunkDB is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    SkunkDB is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with SkunkDB; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <skunk_app.h>
#include <skunk.h>

#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <string.h>

char *get_value(char *data, skunk_ofs_t len)
{
	char *x=malloc(len+1);

	if ( !x ) return NULL;
	
	memset(x, 0, len+1);
	memcpy(x, data, len);
	
	return x;
}

int main ( int argc, char **argv ) 
{
	int i;
	int fd;
	char *map;
	struct stat st;
	p_skunk_table p;
	skunk_ofs_t data=0,count=0;
	skunk_ofs_t min_fill=0, max_fill=0;
	if ( argc < 2 ) {
		printf("Usage: %s <sdb_file>\n", argv[0]);
		return SKUNK_ERR;
	}
	
	if ( (fd=open(argv[1], O_RDONLY)) < 0 ) {
		perror("open");
		return SKUNK_ERR;
	}

	if ( (fstat(fd, &st)) < 0 ) {
		perror("fstat");
		close(fd);
		return SKUNK_ERR;
	}

	if ( st.st_size < SKUNK_DIRECTORY_SIZE ) {
		close(fd);
		return SKUNK_ERR;
	}
	
	if ( !(map=mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0)) )
	{
		perror("mmap");
		close(fd);
		return SKUNK_ERR;
	}

	for (i=0; i<SKUNK_DIRECTORY_MOD; i++) {
		p=(p_skunk_table)(((long)map)+(i*sizeof(skunk_table_t)));
		if ( i==0 ) min_fill=p->len;

		count+=p->len;
		if ( p->len > max_fill ) max_fill=p->len;
		if ( p->len < min_fill ) min_fill=p->len;
	}
	/*
	printf("STATISTICS: optimal=%u min=%u max=%u\n\n",
		count/SKUNK_DIRECTORY_MOD, min_fill, max_fill);
	*/

	data=SKUNK_DIRECTORY_MOD*sizeof(skunk_table_t);
	for(i=0; i<count; i++) {
		skunkrec_t r;
		p_skunk_bucket b;

		b=(p_skunk_bucket)(((long)map) + data);
		if ( !b->ofs ) goto game_over;
		
		r.key_len=b->klen;
		r.val_len=b->vlen;
		
		r.key=get_value(
			(char *)(((skunk_ofs_t )map)+b->ofs),
			r.key_len);
		r.val=get_value(
			(char *)(((skunk_ofs_t )map)+b->ofs+r.key_len),
			r.val_len);
		
		/*
		printf("[0x%.8x:%u]: %s\t -> %s\n",
			b->hash, b->ofs, r.key, r.val);
		*/
		
		printf("%s:%s\n", r.key, r.val);
		
		if ( r.key ) free(r.key);
		if ( r.val ) free(r.val);
		
game_over:
		data+=sizeof(skunk_bucket_t);

		/* quick and dirty check */
		if ( data > st.st_size ) break;
	}
	
	munmap(map, st.st_size);
	close(fd);
	exit(SKUNK_OK);
}
